# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import _, api, models, fields, exceptions
from openerp.tools.safe_eval import safe_eval
from openerp.addons import decimal_precision as dp
from copy import deepcopy

import logging

logger = logging.getLogger(__name__)


class FinancingRule(models.Model):
    _name = 'financing.rule'

    python_condition = fields.Text(required=True)
    priority = fields.Integer(required=True)

    @api.constrains
    def _check_priority(self):
        for fr in self:
            if fr.priority < 0:
                raise exceptions.ValidationError(_('Cannot set a priority lower than zero'))

    @api.multi
    def check_applicable(self, evaldict={}, pid=None):
        scope = deepcopy(evaldict)
        scope['current_product_id'] = pid
        logger.debug('check_applicable scope %s' % scope)
        res = False
        try:
            safe_eval(self.python_condition, scope, mode='exec', nocopy=True)
        except Exception as e:
            logger.warning('Error while computing rule: %s' % e)
        else:
            res = 'result' in scope and bool(scope['result']) or False
            logger.debug('Evaluated to %s' % res)
        return res


class FinancingConfig(models.Model):
    _name = 'financing.config'
    _description = 'Financing General Configuration'
    _order = "customer_type desc, quota_qty"
    _rec_name = "quota_qty"

    @api.constrains('card_ids', 'journal_id')
    def _check_card_in_journal(self):
        """
        1. A card that is not of the journal cannot be added
        """
        for fc in self:
            if fc.card_ids:
                journal_cards = fc.journal_id.card_ids
                diff = fc.card_ids - journal_cards
                if diff:
                    raise exceptions.ValidationError(_('Some cards are not present on journal: %s') % [c.name for c in diff])

    @api.model
    def get_data(self, product_ids, journal_id, partner_id, pricelist_id, pos_config_id):
        """
        :returns: dict {'quota_1': coeff, 'quota_2': coeff, }
        :raises: validation error on problems. Causes the pos not to allow the payment
        """
        parameters_str = '[get_data]\n => product_ids=%s; journal_id=%s; partner_id=%s, pricelist_id=%s; pos_config_id=%s' % (
            product_ids, journal_id, partner_id, pricelist_id, pos_config_id
        )
        q = """
            SELECT component_id
            FROM product_combo_rel
            WHERE product_id IN %(product)s;
        """
        q_p = {
            'product': tuple(product_ids),
        }
        self._cr.execute(q, q_p)
        if self._cr.rowcount:
            components = map(lambda x: x[0], self._cr.fetchall())
            product_ids = [i for i in product_ids if i not in components]
        res = self.perform_search(product_ids, journal_id, partner_id, pricelist_id, pos_config_id)
        logger.info('%s\n <= %s' % (parameters_str, res))
        return res

    @api.model
    def get_configured_quotas_for_journal(self, journal_id):
        """
        Analize configured financing for a journal. At least it should have a card associated.
        :: In :: journal_id (financial journal)
        :: Out New :: {quota_x: 0, quota_z: 0}
        """
        q = """
            SELECT fc.journal_id, fc.quota_qty
            FROM financing_config fc
            WHERE fc.journal_id=%(journal_id)s
        """
        q_p = {'journal_id': journal_id}
        self._cr.execute(q, q_p)
        journal_cards_res = self._cr.fetchall()
        res = {}
        for journal_id, quota_qty in journal_cards_res:
            res[quota_qty] = 0
        logger.debug('In: journal_id %s; Out: %s' % (journal_id, res))
        return res

    @api.model
    def check_disabled(self, fc_list=[]):
        print fc_list
        is_disabled = any([i['disabled'] for i in fc_list])
        if is_disabled:
            raise exceptions.ValidationError(_('The financing has been disabled: %s' % fc_list))

    @api.model
    def check_monthly_rate(self, fc_list=[]):
        print fc_list
        rates = set([i['monthly_rate'] for i in fc_list])
        if len(rates) > 1:
            raise exceptions.ValidationError(_('More than one rate available: %s') % fc_list)

    @api.model
    def get_best_config(self, configs, pid):
        rule_obj = self.env['financing.rule']
        rules = rule_obj.search([], order='priority')
        best_match = None
        best_config = []
        for config in configs:
            for rule in rules:
                if rule.check_applicable(config, pid):
                    if best_match is None or rule.priority <= best_match:
                        if best_match == rule.priority:
                            best_config.append(config)
                        else:
                            best_match = rule.priority
                            best_config = [config]
                        break
        return best_config

    @api.model
    def lookup_financing(
            self, product_ids=[], journal_id=False, customer_type='na',
            pricelist_id=False, pos_config_id=False, quota_qty=False):
        """
        Algorithm to determine the coeff
        """
        try:
            assert product_ids, 'Expected at least one product'
            assert journal_id, 'Expected a journal'
            assert quota_qty, 'Expected a quota_qty'
            assert customer_type != 'na', 'Customer Type is required to perform lookup (cannot be Non-Applicable)'
        except AssertionError as e:
            raise exceptions.ValidationError(e.message)
        parameters_str = '=> product_ids=%s; journal_id=%s; customer_type=%s; pricelist_id=%s; pos_config_id=%s; quota_qty=%s' % (
            product_ids, journal_id, customer_type, pricelist_id, pos_config_id, quota_qty
        )
        logger.debug('lookup_financing %s' % parameters_str)
        return_fcid = self._context.get('return_fcid', False)
        coeff = 1
        # Step 1: Try full match customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        q = """
            SELECT
                *
            FROM financing_report
            WHERE
               quota_qty=%(quota_qty)s
               AND journal_id=%(journal_id)s
               AND (customer_type=%(customer_type)s OR customer_type IS NULL)
               AND (pos_id=%(pos_id)s OR pos_id IS NULL)
               AND (pricelist_id=%(pricelist_id)s OR pricelist_id IS NULL)
               AND (product_id IN %(product_ids)s OR product_id IS NULL)
        """
        q_p = {
            'quota_qty': quota_qty,
            'customer_type': customer_type,
            'journal_id': journal_id,
            'pos_id': pos_config_id,
            'pricelist_id': pricelist_id,
            'product_ids': tuple(product_ids),
        }
        self._cr.execute(q, q_p)
        q_res = self._cr.dictfetchall()
        if not q_res:
            logger.debug('Query was not successfull...\n %s' % self._cr.mogrify(q, q_p))
        res = []
        for pid in product_ids:
            res += self.get_best_config(q_res, pid)

        if res:
            self.check_disabled(res)
            self.check_monthly_rate(res)
            coeff = res[0]['coeff']
            logger.debug('Found Coefficient! \n In: %s\n Out: %s' % (parameters_str, coeff))
            if return_fcid:
                return (coeff, res[0]['fc_id'])
            else:
                return coeff
        else:
            # No results: raise exception
            logger.warning('Cannot found any available coefficient for parameters: \n ' + parameters_str)
            raise exceptions.ValidationError(_('No available financing'))

    @api.model
    def perform_search(self, product_ids, journal_id, partner_id, pricelist_id, pos_config_id):
        """
        Lookups financing config of coeff available for each quota and card of payment meth (journal).
            ~ Algorithm ~
        1. Get all quotas available for journal and its card(s)
        2. For each card & quota lookup for coefficient. But if total_rate differs for same card, send error in card. [ lookup_financing ]
            2.1 customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
            2.2 customer_type & journal_id & card_id & pos_config_id & product_ids
            2.3 customer_type & journal_id & pos_config_id & product_ids
            2.4 customer_type & journal_id & product_ids
            2.5 customer_type == 'na' & journal_id & product_ids
            2.6 customer_type & journal_id
            2.7 customer_type == 'na' & journal_id
            No coeff? raise!
        """
        # Step 1
        quotas_dict = self.get_configured_quotas_for_journal(journal_id)
        # ^- {'quota1': 0, 'quota2': 0, ...}
        ctype_ctx = self._context.get('customer_type')
        customer_type = ctype_ctx or self.env['res.partner'].browse(partner_id).customer_type
        # Step 2.1
        for quota_qty, coeff in quotas_dict.items():
            try:
                coeff = self.lookup_financing(
                    product_ids=product_ids,
                    journal_id=journal_id,
                    customer_type=customer_type,
                    pricelist_id=pricelist_id,
                    pos_config_id=pos_config_id,
                    quota_qty=quota_qty,
                )
            except Exception as e:
                logger.warning(e)
                logger.exception('Ommiting quota_qty %s for journal %s' % (quota_qty, journal_id))
                quotas_dict.pop(quota_qty)
            else:
                quotas_dict[quota_qty] = coeff

        return quotas_dict
        # ^- {quota_qty1: coeff, quota_qty2: coeff}

    @api.depends("quota_qty", "monthly_rate")
    def compute_total_rate(self):
        for pc in self:
            try:
                pc.total_rate = int(pc.quota_qty) * pc.monthly_rate
                pc.coeff = 1 + pc.total_rate
            except ValueError:
                pc.total_rate = 0
                pc.coeff = 1

    quota_qty = fields.Integer(string='Quota Qty.', required=True, default=0)
    monthly_rate = fields.Float(string='Monthly Rate', required=True, default=0,
                                digits=dp.get_precision('Financing Config.'))
    total_rate = fields.Float(string='Total Rate', compute="compute_total_rate",
                              digits=dp.get_precision('Financing Config.'))
    coeff = fields.Float(string='Coefficient', compute="compute_total_rate",
                         digits=dp.get_precision('Financing Config.'), store=True)
    customer_type = fields.Selection(
        [
            ("associated", "Associated"),
            ("non_associated", "Non-associated")
        ],
        string="Customer type",
        required=False,
        help="If non-applicable this financing is defined for associated & non-associated"
    )
    pos_config_ids = fields.Many2many('pos.config', 'financing_pos_rel', 'financing_id', 'pos_id', string='PoS')
    journal_id = fields.Many2one('account.journal', required=True, string='Payment Method', domain=[('pos_payment_type', '=', 'financial')])
    card_ids = fields.Many2many('payment.card', 'financing_card_rel', 'financing_id', 'card_id', string='Card(s)')
    product_ids = fields.Many2many('product.product', 'financing_product_rel', 'financing_id', 'product_id', string='Product(s)', help='If set this financing applies to specific product(s)')
    pricelist_ids = fields.Many2many('product.pricelist', 'financing_pricelist_rel', 'financing_id', 'pricelist_id', string='Pricelist(s)', help='If set this financing only will be applied when selling with this pricelist(s)', domain=[('type', '=', 'sale')])
    category_ids = fields.Many2many('product.category', 'financing_prod_categ_rel', 'financing_id',
                                    'categ_id', string='Categories')
    disabled = fields.Boolean('Disabled?', help='If set this financing is unapplicable')
    date_start = fields.Date('Date Start')
    date_stop = fields.Date('Date Stop')

    _sql_constraints = [
        (
            "check_quota_qty_positive",
            "CHECK(quota_qty > 0)",
            _("Quota Qty. must be > 0!")
        ),
        (
            "check_monthly_rate_positive",
            "CHECK(NOT monthly_rate < 0)",
            _("Monthly Rate must be >= 0!")
        ),
        (
            "check_monthly_rate_not_more_than_1",
            "CHECK(NOT monthly_rate >= 1)",
            _("Monthly Rate must not be >= 1!")
        ),
    ]
