# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (C) 2017-TODAY Eynes (<http://www.e-mips.com.ar>)
#
#    This is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp import models, fields, api, _, exceptions, http
from openerp import SUPERUSER_ID
from lxml import objectify
from unidecode import unidecode
from .wsutils import WSCPZ
import logging
from pprint import pformat as pf
from lxml.objectify import ObjectifiedElement

_logger = logging.getLogger(__name__)


class ResPartner(models.Model):
    _inherit = 'res.partner'

    customer_type = fields.Selection(
        [
            ("associated", "Associated"),
            ("non_associated", "Non-associated"),
        ],
        string="Customer type",
        default="associated",
    )
    credipaz_card_additional = fields.Char('Credipaz Card Additional')
    ignore_cpz_ws = fields.Boolean('Ignore CPZ WS Information Retrieval?')

    _sql_constraints = [
        ('ref_uniq', 'unique(ref)', 'Reference must be unique!'),
    ]

    @api.multi
    def unlink(self):
        if self.env.user.id != SUPERUSER_ID:
            raise exceptions.ValidationError(_('Only the SUPERUSER can delete partners.'))
        super(ResPartner, self).unlink()

    @api.model
    def do_query_ws(self, reference, raise_on_error=True, mode='1'):
        """
        mode:
            1: Account
            2: DNI
        Returns this info: {'disponible': '0', 'error': '0', 'error_msg': '', 'fecha_consulta': '2017-06-30 18:09:21', 'puede_operar': '1', 'puntos': '0', 'socio': '0'}
        """
        wscpz = WSCPZ(self.env)
        ddict = {
            'ConsultarDisponible': {
                'sUsuario': wscpz.ws_user,
                'sPassword': wscpz.ws_pass,
                'sPin': wscpz.ws_pin,
                'sTipoConsulta': mode,
                'sValor': reference,
            }
        }
        wscpz.add(ddict, no_check='all')
        wsres = wscpz.request('ConsultarDisponible')
        res_obj = objectify.fromstring(unidecode(wsres))
        err = False
        if raise_on_error:
            try:
                err = str(res_obj.RegistroSalida.Error)
                if err != '0':
                    raise exceptions.ValidationError('%s (%s)' % (err, res_obj.RegistroSalida.ErrorMensaje))
            except AttributeError:
                pass
        if 'RegistroSalida' not in res_obj.__dict__:
            raise exceptions.ValidationError('Mismatch in ws response.')
        return res_obj.RegistroSalida

    @api.multi
    def get_webservice_data(self):
        self.ensure_one()
        if (not self.ref and not self.vat) or self.ignore_cpz_ws:
            # Does not have a credipaz account, so pricelist cannot be set
            return {
                'partner_id': self.id,
                'available_points': 0,
                'available_credit': 0,
                'ok': False,
                'customer_type': 'non_associated',
            }
        if self.ref:
            wsres = self.do_query_ws(self.ref)
        else:
            wsres = self.do_query_ws(self.vat, mode='2')
        _logger.info('Credipaz WS Respose\n%s' % pf(wsres.__dict__))
        ctype = 'associated' if wsres['SocioRedondo'] != 0 else 'non_associated'
        res = {
            'partner_id': self.id,
            'available_points': wsres['Puntos'],
            'available_credit': wsres['Disponible'],
            'customer_type': ctype,
            'ok': True if wsres['PuedeOperar'] == 0 else False,
        }
        vals = {}
        if not self.ref:
            vals['ref'] = wsres['Cuenta']
        vals['customer_type'] = ctype
        self.sudo().write(vals)
        cpz_code = int(wsres.get('Cpzcode', '99'))
        config_id = self._context.get('config_id')
        cpz_code = '98' if ctype == 'associated' else '99'
        pricelist_id = self.env['product.pricelist'].get_pricelist(config_id, cpz_code)
        if pricelist_id:
            res['pricelist_id'] = pricelist_id
        return {k: str(v) if not isinstance(v, bool) else v for k, v in res.items()}

    @api.model
    def do_custom_check(self, partner):
        """
        Checks if partner data is already fullfilled
        """
        # Custom check
        req_fields = {
            'street': 'Calle',
            'city': 'Ciudad',
            'zip': 'CP',
            'country_id': 'Pais',
            'phone': 'Telefono',
            'email': 'Correo electronico',
            'vat': 'Documento',
        }
        missing_fields = list()
        for k, v in req_fields.items():
            if not partner.get(k):
                missing_fields.append(v)
        if missing_fields:
            missing_fields_str = ', '.join(missing_fields)
            raise exceptions.ValidationError(_('%s requerido(s)') % missing_fields_str)

    @api.model
    def create_from_ui(self, partner):
        """ Get data from WS and set it to partner."""
        try:  # Sanitize account position
            partner.update({'property_account_position': int(partner['property_account_position'])})
        except Exception:
            partner.pop('property_account_position', None)
        # No fiscal position->Set it to consumidor final
        fp_cons = self.env.ref('l10n_ar_point_of_sale.fiscal_position_final_cons')
        if not partner.get('property_account_position', None):
            partner['property_account_position'] = fp_cons.id

        if partner.get('id', False):  # Write
            raise exceptions.ValidationError(_('Cannot modify partner from PoS UI.'))
        else:  # Create
            partner_acc_ref = partner.get('ref')
            partner_vat_ref = partner.get('vat')
            for k, v in partner.items():
                if v == '':
                    partner.pop(k)
            if (not partner_acc_ref) and (not partner['name']):
                raise exceptions.ValidationError(_('You need to provide a name or account reference for customer creation'))
            if (partner_acc_ref or partner_vat_ref) and partner['property_account_position'] == fp_cons.id:  # Contact ws and use this data instead of what comes by UI for contact creation
                if partner_acc_ref:
                    ws_data = self.do_query_ws(partner_acc_ref, raise_on_error=False).__dict__
                else:
                    ws_data = self.do_query_ws(partner_vat_ref, raise_on_error=False, mode='2').__dict__
                _logger.info('Credipaz WS Respose\n%s' % ws_data)
                if ws_data['Error'].__str__() == '0':
                    fp_ws_datas = ws_data.get('PosicionFiscal')
                    if fp_ws_datas:
                        fp_id = self.env['account.fiscal.position'].search([('name', 'ilike', fp_ws_datas)]).id or self.env.ref('l10n_ar_point_of_sale.fiscal_position_final_cons').id
                    else:
                        fp_id = self.env.ref('l10n_ar_point_of_sale.fiscal_position_final_cons').id
                    state_obj = self.env['res.country.state']
                    co_ws_datas = str(ws_data.get('Provincia', '')).strip()
                    state_id = False
                    if co_ws_datas:
                        state_id = state_obj.search([('name', '=ilike', co_ws_datas)], limit=1).id
                    dt_ws_datas = ws_data.get('TipoDocumento')
                    if dt_ws_datas:
                        dt_id = self.env['res.document.type'].search([('name', 'ilike', dt_ws_datas)]).id
                    else:
                        dt_id = self.env.ref('base_vat_ar.document_dni').id
                        # raise exceptions.ValidationError(_('You need to provide a document type for customer creation'))
                    ctype = 'associated' if ws_data.get('SocioRedondo') != 0 else 'non_associated'
                    partner = {
                        'street': ws_data.get('Direccion') or partner.get('street', ''),
                        'city': ws_data.get('Localidad') or partner.get('city', ''),
                        'name': ws_data.get('Nombre') or partner.get('name', ''),
                        'vat': ws_data.get('NroDocumento') or partner.get('vat', ''),
                        'property_account_position': fp_id,
                        'state_id': state_id,
                        'document_type_id': dt_id,
                        'customer_type': ctype,
                        # 'ref': partner_acc_ref,  # ws_data.get('Cuenta', '')
                        'country_id': self.env.ref('base.ar').id,
                        'credipaz_card_additional': ws_data.get('Adicional', ''),
                    }
                    if partner_acc_ref:
                        partner['ref'] = partner_acc_ref
                else:  # Error in ws return: No data to retrieve
                    raise exceptions.ValidationError(_('Invalid account reference, maybe is a spelling mistake or the customer is not present in WS'))
            else:
                partner.pop('ref', None)  # Remove ref to prevent empty string as reference

                # Ensure required fields are complete
                self.do_custom_check(partner)

                if not partner['property_account_position']:
                    cf_fiscal_position = self.env.ref('l10n_ar_point_of_sale.fiscal_position_final_cons')
                    partner['property_account_position'] = cf_fiscal_position.id
                if not partner['document_type_id']:
                    cf_document_type = self.env.ref('base_vat_ar.document_dni')
                    partner['document_type_id'] = cf_document_type.id
                partner['customer_type'] = 'non_associated'
                partner['ignore_cpz_ws'] = True
        res = super(ResPartner, self).create_from_ui(partner)
        return res
