# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2018 E-MIPS (http://www.e-mips.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.exceptions import except_orm, ValidationError
from openerp.tests.common import TransactionCase
from openerp import fields
from datetime import datetime, timedelta, date
from tempfile import TemporaryFile
import logging


class TestFinancing(TransactionCase):

    def tearDown(self):
        super(TestFinancing, self).tearDown()

    def setUp(self):
        super(TestFinancing, self).setUp()
        self.cr.execute('delete from financing_config')
        translate_logger = logging.getLogger('openerp.tools.translate')
        translate_logger.setLevel('INFO')  # mute translations
        self.fc_model = self.env['financing.config']
        self.journal = self.browse_ref('financing.card_journal01')
        self.pl0 = self.browse_ref('product.list0')
        self.pl1 = self.browse_ref('financing.list1')
        self.card01 = self.browse_ref('financing.card01')
        self.card02 = self.browse_ref('financing.card02')
        self.pos_config_main = self.browse_ref('point_of_sale.pos_config_main')
        self.pos_config_main2 = self.ref('financing.pos_config_main2')

    def test_0a00_no_financing_config(self):
        """
        No financing.config available
        """
        with self.assertRaises(ValidationError) as e:
            self.fc_model.lookup_financing(
                product_ids=[self.browse_ref('product.product_product_4').id],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=1,
            )
        self.assertIn('No available financing', e.exception.value)

    def test_0a01_bad_parameters_lookup_financing(self):
        """
        Send bad parameters to lookup should raise errors
        """
        # Not passing any products
        with self.assertRaises(ValidationError) as e:
            self.fc_model.lookup_financing(
                product_ids=[],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=1,
            )
        self.assertIn('Expected at least one product', e.exception.value)
        # Not passing journal
        with self.assertRaises(ValidationError) as e:
            self.fc_model.lookup_financing(
                product_ids=[self.browse_ref('product.product_product_4').id],
                journal_id=False,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=1,
            )
        self.assertIn('Expected a journal', e.exception.value)
        # Not passing quota_qty
        with self.assertRaises(ValidationError) as e:
            self.fc_model.lookup_financing(
                product_ids=[self.browse_ref('product.product_product_4').id],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=False,
            )
        self.assertIn('Expected a quota_qty', e.exception.value)

    def test_0a02_simple_coeff_1q0mr(self):
        """
        Coefficient Retrieval
        - Journal & all customer_types: 1 quota 0 monthly rate
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': False,
            'quota_qty': 1,
            'monthly_rate': 0,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.browse_ref('product.product_product_4').id],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1)

    def test_0a03_simple_coeff_1q01mr(self):
        """
        Coefficient Retrieval
        - Journal & all customer_types: 1 quota 0.1 monthly rate
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.browse_ref('product.product_product_4').id],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.1)

    def test_0a04_simple_coeff_12q0mr(self):
        """
        Coefficient Retrieval
        - Journal & all customer_types: 12 quota 0 monthly rate
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 12,
            'monthly_rate': 0,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.browse_ref('product.product_product_4').id],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=12,
        )
        self.assertEqual(coeff, 1)

    def test_0a05_simple_coeff_12q01mr(self):
        """
        Coefficient Retrieval
        - Journal & all customer_types: 12 quota 0.1 monthly rate
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 12,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.browse_ref('product.product_product_4').id],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=12,
        )
        self.assertEqual(coeff, 2.2)

    def test_0a06_simple_coeff_exception12q01and02mr(self):
        """
        Coefficient Retrieval
        - Journal & all customer_types: 12 quota 0.1 & 0.2 monthly rate
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 12,
            'monthly_rate': 0.1,
        })
        fc02 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 12,
            'monthly_rate': 0.2,
        })
        # If two financings collides with different monthly_rates, exception!
        with self.assertRaises(ValidationError) as e:
            # import ipdb; ipdb.set_trace()
            coeff = self.fc_model.lookup_financing(
                product_ids=[self.browse_ref('product.product_product_4').id],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=12,
            )
        self.assertIn('More than one rate available', e.exception.value)
        # If disabled one of the colliding financing, all are disabled, exception!
        fc02.disabled = True
        with self.assertRaises(ValidationError) as e:
            coeff = self.fc_model.lookup_financing(
                product_ids=[self.browse_ref('product.product_product_4').id],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=12,
            )
        self.assertIn('The financing has been disabled', e.exception.value)

    def test_1a01_simple_retrieval(self):
        """
        Coefficient Retrieval
        - Journal & all customer_types
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 1,
            'monthly_rate': 0,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.browse_ref('product.product_product_4').id],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1)
        fc01.monthly_rate = 0.1
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.browse_ref('product.product_product_4').id],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.1)

    def test_1a02_simple_retrieval_exception(self):
        """
        Configured journal & non_associated financing
        """
        fc01 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'non_associated',
            'quota_qty': 1,
            'monthly_rate': 0,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.browse_ref('product.product_product_4').id],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1)
        with self.assertRaises(ValidationError) as e:
            coeff = self.fc_model.lookup_financing(
                product_ids=[self.browse_ref('product.product_product_4').id],
                journal_id=self.journal.id,
                customer_type='associated',
                pricelist_id=self.pl0.id,
                pos_config_id=self.pos_config_main.id,
                card_id=self.card01.id,
                quota_qty=1,
            )
        self.assertIn('No available financing', e.exception.value)


class TestFinancingCoeffStep1(TransactionCase):
    def tearDown(self):
        super(TestFinancingCoeffStep1, self).tearDown()

    def setUp(self):
        super(TestFinancingCoeffStep1, self).setUp()
        self.cr.execute('delete from financing_config')
        translate_logger = logging.getLogger('openerp.tools.translate')
        translate_logger.setLevel('INFO')  # mute translations
        self.fc_model = self.env['financing.config']
        self.journal = self.browse_ref('financing.card_journal01')
        self.pl0 = self.browse_ref('product.list0')
        self.pl1 = self.browse_ref('financing.list1')
        self.card01 = self.browse_ref('financing.card01')
        self.card02 = self.browse_ref('financing.card02')
        self.pos_config_main = self.browse_ref('point_of_sale.pos_config_main')
        self.pos_config_main2 = self.ref('financing.pos_config_main2')
        self.fcgeneric_ass = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'associated',
            'quota_qty': 1,
            'monthly_rate': 0.4,
        })
        self.fcgeneric_nonass = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'non_associated',
            'quota_qty': 1,
            'monthly_rate': 0.5,
        })
        self.fcgeneric_na = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 1,
            'monthly_rate': 0.7,
        })

    def test_1lf01a(self):
        """
        Lookup coeff: Step 1.
        [2.1] customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        1. Try full match for retrieving coeff with one generic defined for all non_associated that matches for another product
        """
        fc01 = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('product.list0')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.5,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_4')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.5)  # Retrieve the fc01

    def test_1lf01b(self):
        """
        Lookup coeff: Step 1.
        [2.1] customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        1. Trying to match other product should fallback on fcgeneric
        """
        fc01 = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('product.list0')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_5')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.5)  # Retrieve the generic for non_associated

    def test_1lf01c(self):
        """
        lookup coeff: step 1.
        [2.1] customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        1. trying to match other card should fallback on fcgeneric
        """
        fc01 = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('product.list0')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_4')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card02.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.5)  # retrieve the generic for non_associated

    def test_1lf01d(self):
        """
        lookup coeff: step 1.
        [2.1] customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        1. trying to match other pricelist should fallback on fcgeneric
        """
        fc01 = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('financing.list1')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_4')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card02.id,
            quota_qty=1,
        )
        assert coeff == 1.5  # retrieve the generic for non_associated

    def test_1lf01e(self):
        """
        lookup coeff: step 1.
        [2.1] customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        1. trying to match other pos_config_id should fallback on fcgeneric
        """
        fc01 = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('financing.pos_config_main2')])],
            'pricelist_ids': [(6, 0, [self.ref('financing.list1')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_4')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card02.id,
            quota_qty=1,
        )
        assert coeff == 1.5  # retrieve the generic for non_associated

    def test_1lf01f(self):
        """
        lookup coeff: step 1.
        [2.1] customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        1. trying to match other journal should fallback on fcgeneric
        """
        fc01 = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card02')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('financing.list1')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_4')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card02.id,
            quota_qty=1,
        )
        assert coeff == 1.5  # retrieve the generic for non_associated

    def test_step1_different_customer_type_should_fallback_on_generic(self):
        """
        lookup coeff: step 1.
        [2.1] customer_type & journal_id & card_id & pos_config_id & pricelist_id & product_ids
        1. trying to match other customer_type should fallback on fcgeneric
        """
        fc01 = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card02')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('financing.list1')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_4')],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card02.id,
            quota_qty=1,
        )
        assert coeff == 1.4  # retrieve the generic for associated


class TestFinancingCoeffStep2(TransactionCase):
    def tearDown(self):
        super(TestFinancingCoeffStep2, self).tearDown()

    def setUp(self):
        super(TestFinancingCoeffStep2, self).setUp()
        self.cr.execute('delete from financing_config')
        translate_logger = logging.getLogger('openerp.tools.translate')
        translate_logger.setLevel('INFO')  # mute translations
        self.fc_model = self.env['financing.config']
        self.journal = self.browse_ref('financing.card_journal01')
        self.pl0 = self.browse_ref('product.list0')
        self.pl1 = self.browse_ref('financing.list1')
        self.card01 = self.browse_ref('financing.card01')
        self.card02 = self.browse_ref('financing.card02')
        self.pos_config_main = self.browse_ref('point_of_sale.pos_config_main')
        self.pos_config_main2 = self.ref('financing.pos_config_main2')
        self.fcgeneric_ass = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'associated',
            'quota_qty': 1,
            'monthly_rate': 0.4,
        })
        self.fcgeneric_nonass = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'non_associated',
            'quota_qty': 1,
            'monthly_rate': 0.5,
        })
        self.fcgeneric_na = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'quota_qty': 1,
            'monthly_rate': 0.7,
        })
        # Configured taking into account step 1
        self.fc01_nonass = self.fc_model.create({
            'customer_type': 'non_associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('product.list0')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.1,
        })
        self.fc02_ass = self.fc_model.create({
            'customer_type': 'associated',
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('product.list0')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.2,
        })
        self.fc02 = self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': [(6, 0, [self.ref('product.list0')])],
            'product_ids': [(6, 0, [self.ref('product.product_product_4')])],
            'quota_qty': 1,
            'monthly_rate': 0.3,
        })

    def test_step2_01_match_for_generics_customers(self):
        """
        Lookup coeff: Step 2.
        [2.2] customer_type & journal_id & card_id & pos_config_id & product_ids
        1. Configure a rule without pricelist
        1. Lookup using a different product set in step in step 1 (setup)
        """
        self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'pos_config_ids': [(6, 0, [self.ref('point_of_sale.pos_config_main')])],
            'pricelist_ids': False,
            'product_ids': [(6, 0, [self.ref('product.product_product_5')])],
            'quota_qty': 1,
            'monthly_rate': 0.9,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_5')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.9)  # Retrieve the fc
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_5')],
            journal_id=self.journal.id,
            customer_type='non_associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.9)  # Retrieve the fc

    def test_shadowing_generic_config_by_applying_pricelist_and_card(self):
        """
        1. Define two configurations for same quotas & journal & customer condition with different rates
            1.1 One is done in setup
            1.2 Second with a card defined and pricelist defined
        Expect as result the second financing config
        """
        self.fc_model.create({
            'journal_id': self.ref('financing.card_journal01'),
            'customer_type': 'associated',
            'pos_config_ids': False,
            'card_ids': [(6, 0, [self.ref('financing.card01')])],
            'product_ids': False,
            'pricelist_ids': [(6, 0, [self.ref('product.list0')])],
            'quota_qty': 1,
            'monthly_rate': 0.5,
        })
        coeff = self.fc_model.lookup_financing(
            product_ids=[self.ref('product.product_product_5')],
            journal_id=self.journal.id,
            customer_type='associated',
            pricelist_id=self.pl0.id,
            pos_config_id=self.pos_config_main.id,
            card_id=self.card01.id,
            quota_qty=1,
        )
        self.assertEqual(coeff, 1.5)  # Retrieve the fc

# control this issue
# 2018-10-02 12:23:31,245 26202 INFO credipaz openerp.addons.financing.models.financing: => product_ids=[388]; journal_id=47; customer_type=associated; pricelist_id=9; pos_config_id=25; card_id=4; quota_qty=6
#2018-10-02 12:23:31,250 26202 WARNING credipaz openerp.addons.financing.models.financing: ('ValidateError', u"Mas de una tasa mensual disponible: [(6, 0.029, 1.174, None, u'associated', 47, None, None, None, None, None), (6, 0.3 , 2.8, False, u'associated', 47, 4, None, 9, None, None)]")
#2018-10-02 12:23:31,251 26202 WARNING credipaz openerp.addons.financing.models.financing: Omitting quota_qty 6 for card_id 4: ('ValidateError', u"Mas de una tasa mensual disponible: [(6, 0.029, 1.174, None, u'associated', 47, No ne, None, None, None, None), (6, 0.3, 2.8, False, u'associated', 47, 4, None, 9, None, None)]")
