# coding: utf-8

from openerp import models, api, fields, exceptions, _
from openerp.addons.decimal_precision import decimal_precision as dp
import logging

logger = logging.getLogger(__name__)


class FinancingPriceQuery(models.TransientModel):
    _name = "financing.price.query"
    _description = "Query the final price for each financing"

    product_ids = fields.Many2many('product.product', 'fp_product_rel', 'fp_id', 'product_id', string='Product(s)')
    pos_config_id = fields.Many2one('pos.config', string='PoS')
    pricelist_id = fields.Many2one('product.pricelist', string='Pricelist', domain=[('type', '=', 'sale')])
    item_ids = fields.One2many('financing.price.query.lines', 'query_id')

    @api.model
    def default_get(self, fields):
        res = super(FinancingPriceQuery, self).default_get(fields)
        default_pos = self.env.user.pos_config
        if default_pos and 'pos_config_id' in fields:
            res['pos_config_id'] = default_pos.id
            if 'pricelist_id' in fields:
                res['pricelist_id'] = default_pos.pricelist_id.id
        active_model = self.env.context.get('active_model')
        active_ids = self.env.context.get('active_ids')
        if active_model == 'product.template' and active_ids:
            pp_ids = self.env['product.template'].browse(active_ids).mapped('product_variant_ids').ids
            res['product_ids'] = [(6, 0, pp_ids)]
        if active_model == 'product.product' and active_ids:
            res['product_ids'] = [(6, 0, active_ids)]
        return res

    @api.multi
    def generate(self):
        self.ensure_one()
        logger.info('[financing] generate')
        if not self.product_ids:
            raise exceptions.ValidationError(_('No products selected'))
        ctx_ctype = self.env.context.get('customer_type')
        if ctx_ctype:
            customer_types = [ctx_ctype]
        else:
            logger.warning('No customer_type set. Assume export for all pos.')
            customer_types = ['associated', 'non_associated']
        pl_obj = self.env['product.pricelist']
        if not self.pos_config_id:
            logger.warning('No pos_config_id set. Assume export for all pos.')
            pos_ids = self.env['pos.config'].search([])
        else:
            pos_ids = self.pos_config_id
        pos_ids_list = [i.id for i in pos_ids]
        if not self.pricelist_id:
            pricelist_domain = [
                '|',
                ('pos_config_ids', '=', False),
                ('pos_config_ids', 'in', pos_ids_list),
                ('type', '=', 'sale'),
            ]
            logger.warning('No pricelist set, assume export for all related pricelists')
            pricelists = pl_obj.search(pricelist_domain)
        else:
            pricelists = self.pricelist_id
        f_obj = self.env['financing.config']
        res = []
        for pos in pos_ids:
            journals = pos.journal_ids.filtered(lambda x: x.pos_payment_type == 'financial')
            for ctype in customer_types:
                for journal in journals:
                    for product in self.product_ids:
                        for pricelist in pricelists:
                            fdata = f_obj.with_context(customer_type=ctype).get_data([product.id], journal.id, None, pricelist.id, pos.id)
                            fdata_processed = self.process_data(fdata, product.id, journal.id, pricelist.id, pos.id, ctype)
                            res.extend(fdata_processed)

        res_tbi = [(0, 0, fdp) for fdp in res]
        self.write({'item_ids': res_tbi})
        view = self.env.ref('financing.view_financing_price_query_lines_tree_backend')
        ctx = self.env.context.copy()
        ctx['search_default_groupby_easy_filter'] = 1
        return {
                'name': _('Financing Assistent'),
                'res_model': 'financing.price.query.lines',
                'type': 'ir.actions.act_window',
                'views': [(view.id, 'tree')],
                'view_id': view.id,
                'context': ctx,
                'domain': [('query_id', '=', self.ids[0])]
        }

    @api.model
    def process_data(self, fdata, product_id, journal_id, pricelist_id, pos_id, customer_type):
        """Process data returned from financing.config get_data
        ^-- {'quota_1': coeff, 'quota_2': coeff,}
        <- Returns data ready to be inserted on lines
        """
        res = []
        quotas_dict = fdata
        for quota, coeff in quotas_dict.items():
            res_dd = {
                'customer_type': customer_type,
                'product_id': product_id,
                'pricelist_id': pricelist_id,
                'pos_config_id': pos_id,
                'journal_id': journal_id,
                'quota_qty': quota,
                'coeff': coeff,
            }
            res.append(res_dd)
        return res


class FinancingPriceQueryLines(models.TransientModel):
    _name = "financing.price.query.lines"
    _order = "customer_type, quota_qty"

    query_id = fields.Many2one('financing.price.query')
    customer_type = fields.Selection([("associated", "Associated"), ("non_associated", "Non-associated")], string="Customer type")
    product_id = fields.Many2one('product.product', string='Product')
    pricelist_id = fields.Many2one('product.pricelist', string='PriceList')
    pos_config_id = fields.Many2one('pos.config', string='PoS')
    journal_id = fields.Many2one('account.journal', string='Journal')
    coeff = fields.Float('Coefficient', digits=dp.get_precision('Financing Config.'))
    quota_qty = fields.Integer('Quota Qty.')
    list_price = fields.Float('List Price', compute='_compute_price')
    quota_amount = fields.Float('Quota Amount', compute='_compute_price')
    financed_price = fields.Float('Financed Price', compute='_compute_price')

    def read_group(self, cr, uid, domain, fields, groupby, offset=0, limit=None, context=None, orderby=False, lazy=True):
        if 'quota_qty' in fields:
            fields.remove('quota_qty')
        res = super(FinancingPriceQueryLines, self).read_group(
            cr, uid, domain, fields, groupby, offset=offset, limit=limit,
            context=context, orderby=orderby, lazy=lazy)
        return res

    @api.multi
    @api.depends('pricelist_id', 'product_id', 'coeff')
    def _compute_price(self):
        logger.info('[financing] _compute_price')
        for item in self:
            taxes = item.product_id.taxes_id
            list_price = item.product_id.with_context(pricelist=item.pricelist_id.id).price
            info = taxes.compute_all(list_price, 1, force_excluded=True)
            price = info['total_included']
            financed_price = price * item.coeff
            quota_amount = financed_price / item.quota_qty
            item.list_price = price
            item.financed_price = financed_price
            item.quota_amount = quota_amount
